import josx.robotics.*;
import josx.platform.rcx.*;
import java.util.Random;

public class Sumobot1 implements SensorConstants {
    public static void main(String[] args) {
	Sensor.S1.setTypeAndMode(SENSOR_TYPE_LIGHT, SENSOR_MODE_PCT);
	Sensor.S1.activate();
	Sensor.S2.setTypeAndMode(SENSOR_TYPE_TOUCH, SENSOR_MODE_BOOL);
	Sensor.S3.setTypeAndMode(SENSOR_TYPE_TOUCH, SENSOR_MODE_BOOL);

	Behavior b1 = new DriveForward();
	Behavior b2 = new HitFrontBumper();
	Behavior b3 = new HitRearBumper();
	Behavior b4 = new HitLine();
	Behavior[] bs = {b1, b2, b3, b4};
	Arbitrator a = new Arbitrator(bs);
	a.start();
    }
}

class DriveForward implements Behavior {

    public void action() {
	Motor.A.forward();
	Motor.C.forward();
    }

    public void suppress() {
	Motor.A.stop();
	Motor.C.stop();
    }

    public boolean takeControl() {
	return true;
    }
}

class HitFrontBumper implements Behavior {

    public void action() {
	Motor.A.forward();
	Motor.C.forward();
    }

    public void suppress() {
	Motor.A.stop();
	Motor.C.stop();
    }

    public boolean takeControl() {
	return Sensor.S2.readBooleanValue();
    }
}


class HitRearBumper implements Behavior {

    public void action() {
	try {
	    Motor.A.backward();
	    Motor.C.backward();
	    Thread.sleep(500);
	    Motor.A.forward();
	    Motor.C.forward();
	    Thread.sleep(100);
	    Motor.C.stop();
	    Thread.sleep(800);
	} catch (InterruptedException e) {
	}
    }

    public void suppress() {
	Motor.A.stop();
	Motor.C.stop();
    }

    public boolean takeControl() {
	return Sensor.S3.readBooleanValue();
    }
}


class HitLine implements Behavior {
    private Thread actionThread;
    private Random r;

    HitLine() {
	r = new Random();
    }

    public void action() {
	actionThread = Thread.currentThread();
	try {
	    Motor.A.backward();
	    Motor.C.backward();
	    Thread.sleep(300);
	    if (r.nextInt() % 4 != 0) {
		Motor.A.stop();
	    } else {
		Motor.C.stop();
	    }
	    Thread.sleep(800);
	} catch (InterruptedException e) {
	}
    }

    public void suppress() {
	Motor.A.stop();
	Motor.C.stop();
    }

    public boolean takeControl() {
	return Sensor.S1.readValue() < 38;
    }
}
